'''
Checks to see which flux tower sites in Yushu's list are already in the
SMAP L4_C Cal/Val dataset.

Then, create a new HDF5 dataset that will be populated with the data
for each site.
'''

import csv
import datetime
import h5py
import numpy as np

YUSHU_CSV = '/home/arthur.endsley/Workspace/NTSG/SMAP_L4C/applications/Western_Rangelands/Yushu_AmeriFlux_final_sites_2022.csv'
OLD_HDF_FILE = '/anx_lagr3/arthur.endsley/SMAP_L4C/calibration/v6_Y2021/L4_C_tower_site_drivers_NRv9-1_for_356_sites.h5'
NEW_HDF_FILE = '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5'
DATE_START = datetime.date(2000, 1, 1)
DATE_END = datetime.date(2022, 12, 31)
N_DAYS = 8401 # Daily files from 2000-01-01 through 2022-12-31 (8401 days)

def main():
    dates = [DATE_START]
    while dates[-1] < DATE_END:
        dates.append(dates[-1] + datetime.timedelta(days = 1))
    dates = np.array([
        (d.year, d.month, d.day, 0) for d in dates
    ])

    with h5py.File(OLD_HDF_FILE, 'r') as hdf:
        sites = hdf['site_id'][:].tolist()
        sites = list(map(lambda x: x.decode('utf-8'), sites))

    with open(YUSHU_CSV, 'r') as file:
        reader = csv.reader(file)
        yushu_sites = [
            line[1] if reader.line_num > 1 else '' for line in reader
        ]
        file.seek(0) # Go back to top of file
        reader = csv.reader(file)
        yushu_coords = [
            (float(line[-1]), float(line[-2])) if reader.line_num > 1 else ''
            for line in reader
        ]
    yushu_sites.remove('')
    yushu_coords.remove('')

    # This doesn't really matter, we're just going to grab data for all the
    #   sites regardless of whether they're in the SMAP Cal/Val set or not
    missing = set(yushu_sites).difference(sites)
    mask = np.array([name in yushu_sites for name in sites])
    # Assert that everything adds up correctly
    assert len(missing) + sum(mask) == len(yushu_sites)

    # A single array of all site names
    all_sites = np.concatenate(
        (list(missing), np.array(sites)[mask])).tolist()
    all_sites.sort()
    all_coords = np.array([
        yushu_coords[yushu_sites.index(name)] for name in all_sites
    ]).swapaxes(0, 1)

    old = h5py.File(OLD_HDF_FILE, 'r')
    with h5py.File(NEW_HDF_FILE, 'a') as hdf:
        dset = hdf.create_dataset('time', dates.shape, dates.dtype, dates)
        dset.attrs['description'] = '(Year, Month, Day, Hour) for the time axis of data arrays; Hour is not used'

        hdf.create_group('coords')
        dset = hdf.create_dataset('coords/lng_lat', all_coords.shape, np.float32, all_coords)
        dset.attrs['description'] = 'Longitude/ latitude coordinates for each site'

        dt = h5py.string_dtype(encoding = 'utf-8')
        dset = hdf.create_dataset('site_id', (len(all_sites),), dt)
        for i, name in enumerate(all_sites):
            dset[i] = name


if __name__ == '__main__':
    main()
