'''
Extracts MERRA-2 surface meteorology for the eddy covariance tower sites.

The output HDF5 field structure looks like (names with a "/" are groups, all
others are data fields):

    coords/
    MERRA2/
        Tmin
        T2M
        QV2M
        PS
        TS
        SWGDN
    site_id
    time
'''

import datetime
import glob
import os
import re
import warnings
import netCDF4
import h5py
import numpy as np
import xarray as xr
from typing import Sequence
from tqdm import tqdm

EXPECTED_NUM_FILES = 8401 # Daily files from 2000-01-01 through 2022-12-31 (8401 days)
OUTPUT_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5'
MERRA2_DIR = '/anx_lagr4/SMAP/L4C_drivers/MERRA-2'
MERRA2_DATASETS = [ # (dataset, filename template) pairs
    ('tavg1_2d_slv_Nx*', 'MERRA2_???.tavg1_2d_slv_Nx.*.???.nc*'),
    ('tavg1_2d_slv_Nx',  'MERRA2_???.tavg1_2d_slv_Nx.*.???.nc*'),
    ('tavg1_2d_rad_Nx',  'MERRA2_???.tavg1_2d_rad_Nx.*.???.nc*'),
]
MERRA2_FIELDS = {
    'tavg1_2d_slv_Nx*': ('Tmin',),
    'tavg1_2d_slv_Nx':  ('T2M', 'QV2M', 'PS', 'TS'),
    'tavg1_2d_rad_Nx':  ('SWGDN',)
}

# Flip data up/down (ud) or left/right?
#   Argument lats/ lons should be a 1D numpy.ndarray
flip_ud = lambda lats: np.argmax(lats) > np.argmin(lats)
flip_lr = lambda lons: np.argmax(lons) < np.argmin(lons)


def main():
    'Populates the calibration driver dataset with data from MERRA2'
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    if 'MERRA2' not in target_hdf.keys():
        target_hdf.create_group('MERRA2')
    shp = (EXPECTED_NUM_FILES, target_hdf['site_id'].size)
    coords = target_hdf['coords/lng_lat'][:].swapaxes(0, 1)
    coord_arrays = (
        xr.DataArray(coords[:,0], dims = 'points'),
        xr.DataArray(coords[:,1], dims = 'points'),
    )
    for dataset, template in MERRA2_DATASETS:
        print(f'Working on "{dataset}" dataset...')
        file_list = glob.glob(f'{MERRA2_DIR}/{dataset}/{template}')
        assert len(file_list) == EXPECTED_NUM_FILES,\
            f'Did not find {EXPECTED_NUM_FILES} files!'
        file_list.sort() # Get chronological order
        # Create a dataset for each field, including "_daytime" and
        #   "_nightttime" datasets
        nc0 = netCDF4.Dataset(file_list[0])
        for field in MERRA2_FIELDS[dataset]:
            if f'MERRA2/{field}' not in target_hdf.keys():
                dset = target_hdf.create_dataset(
                    f'MERRA2/{field}', shp, np.float32)
                _field = field
                if field == 'Tmin':
                    _field = 'T2M'
                dset.attrs['description'] = nc0.variables[_field].long_name
                dset.attrs['units'] = nc0.variables[_field].units
        nc0.close()
        # Iterate over files, in chronological order; "i" indexes the day
        for i, filename in enumerate(tqdm(file_list)):
            nc = netCDF4.Dataset(filename)
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                data = extract(
                    filename, coord_arrays, MERRA2_FIELDS[dataset])
            for field in data.keys():
                target_hdf[f'MERRA2/{field}'][i] = data[field][0]


def extract(
        filename: str, coords: tuple, fields: Sequence) -> list:
    '''
    Extracts data from a MERRA-2 netCDF4 granule, based on the provided field.

    Parameters
    ----------
    filename : str
    coords : tuple
        A tuple of two xarray.DataArray: (longitude, latitude)
    fields : Sequence
        Sequence of the field names to extract

    Returns
    -------
    dict
        A mappping of (`field_name`, `numpy.ndarray`) pairs
    '''
    nc = xr.open_dataset(filename)
    assert nc['time'].size == 24, 'Unexpected time axis length!'
    # Nearest-neighbour lookup for each site
    x, y = coords
    subset = nc.sel(lon = x, lat = y, method = 'nearest')
    results = []
    for field_name in fields:
        if field_name == 'Tmin':
            tmin = subset['T2M'].values.min(axis = 0)
            results.append((field_name, tmin))
        else:
            arr = subset[field_name].values
            avg = arr.mean(axis = 0) # 24-hour average
            results.append((field_name, arr))
    return dict(results)


if __name__ == '__main__':
    main()
