'''
Samples soil moisture, temperature fields from the the daily average L4SM
Nature Run v10 "tile-space" granules.

The difference between `ORIGIN` and `START_DATE` is that the `ORIGIN` is the
first date in the time series and `START_DATE` is the first date that will be
processed in *this particular run,* allowing for concurrency/ batch runs with
different time intervals.

On Discover:
    module load python/GEOSpyD/Min4.11.0_py3.9

SLURM options:
    --constraint="cas&cssro"
'''

import csv
import os
import pickle
import datetime
import glob
import numpy as np
import h5py
import netCDF4
import dask.bag as db
from functools import partial
from pyl4c.apps.calibration.nature import NatureRunNetCDF4
from pyl4c.ease2 import ease2_from_wgs84

OUTPUT_HDF = '/discover/nobackup/kendsley/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5'
NATURE_RUN_DIR = '/discover/nobackup/projects/gmao/smap/SMAP_Nature/SMAP_Nature_v10/SMAP_Nature_v10.0/output/SMAP_EASEv2_M09_GLOBAL/cat/ens0000/'
NATURE_RUN_TPL = 'Y%d/M%s/SMAP_Nature_v10.0.tavg3_1d_lnr_Nt.%s.nc4'
PICKLE_FILE = '/home/kendsley/20230511_0301.pickle'
FIELDS = [
    ('sm_surface', 'drivers/smsf_vwc'),
    ('sm_rootzone', 'drivers/smrz_vwc'),
    ('soil_temp_layer1', 'drivers/tsoil'),
]
ORIGIN = datetime.datetime(2000, 1, 1) # MUST always be date for first time index
START_DATE = datetime.datetime(2000, 1, 1) # Starting date for time series to extract
END_DATE = datetime.datetime(2007, 12, 31)
TOTAL_NUM_DAYS = 8401 # Daily values from 2000-01-01 through 2022-12-31 (8401 days)
NUM_DAYS = (END_DATE - START_DATE).days
NUM_PROCESSES = 14

def main():
    'Loops through each Nature Run file, sampling fields, writes to HDF5'
    
    with h5py.File(OUTPUT_HDF, 'r') as hdf:
        sites = hdf['site_id'][:].tolist()
        coords = hdf['coords/lng_lat'][:].swapaxes(0, 1).tolist()

    dates = [START_DATE]
    while dates[-1] < END_DATE:
        dates.append(dates[-1] + datetime.timedelta(days = 1))

    # Translate coordinate pairs from WGS84 to EASE-Grid 2.0 row-column
    coords = [ease2_from_wgs84(pair, grid = 'M09') for pair in coords]

    # Output array shape
    shp = (TOTAL_NUM_DAYS, len(sites))
    with h5py.File(OUTPUT_HDF, 'a') as hdf:
        for _, field in FIELDS:
            if field in hdf.keys():
                print('WARNING: Field "%s" already present in output HDF5 dataset!' % field)
            else:
                hdf.create_dataset(field, shp, np.float32, np.full(shp, np.nan))

    # It doesn't matter which we use for this purpose, the first file serves
    nr = NatureRunNetCDF4(
        os.path.join(NATURE_RUN_DIR, NATURE_RUN_TPL % (2000, '01', '20000101')))
    idx = list()
    # Get a 1D list of coordinates in "tile space"
    for coord_pair in coords:
        idx.append(nr.ease2_to_tile(*coord_pair))

    file_list = [
        glob.glob(os.path.join(
            NATURE_RUN_DIR, NATURE_RUN_TPL % (
                ts.year, str(ts.month).zfill(2), ts.strftime('%Y%m%d')))
            ).pop()
        for ts in dates
    ]
    assert len(file_list) > 0, 'No matching files found!'
    print('Found %d files...' % len(file_list))  

    # About 2 minutes for 9 files, file-wise
    bag = db.from_sequence(file_list, npartitions = NUM_PROCESSES)
    merra2_extract = partial(extract, indices = idx, n_sites = len(sites))
    results = bag.map(merra2_extract).compute()

    with open(PICKLE_FILE, 'wb') as file:
        pickle.dump(results, file)

    with h5py.File(OUTPUT_HDF, 'a') as hdf:
        for i, arrays in results:
            for f, pair in enumerate(FIELDS):
                _, field = pair
                hdf[field][i] = arrays[f][:]


def extract(filename, indices, n_sites):
    'Calculates a 24-hour mean for each MERRA-2 field, returns tuple()'
    date = datetime.datetime.strptime(os.path.basename(filename).split('.')[-2], '%Y%m%d')
    ordinal = (date - ORIGIN).days
    results = (ordinal, np.ones((len(FIELDS), n_sites,)) * np.nan)
    with netCDF4.Dataset(filename) as nc:
        for i, pair in enumerate(FIELDS):
            source_field, field = pair
            arr = nc.variables[source_field][:,indices]
            # Calculate daily average across 3-hourly intervals
            if nc.variables[source_field].shape[0] == 8:
                results[1][i,:] = arr.mean(axis = 0).astype(np.float32)
            else:
                results[1][i,:] = arr.astype(np.float32)
    return results


if __name__ == '__main__':
    import fire
    fire.Fire(main)

