'''
Re-doing this using a 3-by-3 subgrid to reduce the risk of sampling empty
pixels. Extracts the MODIS Collection 6.1 MOD15A2H fPAR data for Yushu's
Western Rangelands sites.

/anx_lagr4/MODIS_VIIRS/MOD15A2H/all_granules/Y2019/MOD15A2H.A2019361.h35v10.hdf
'''

import datetime
import glob
import os
import re
import h5py
import numpy as np
import rasterio as rio
from rasterio.warp import reproject, Resampling
from osgeo import gdal
from affine import Affine
from tqdm import tqdm
from pyl4c.lib.modis import TILE_SIZE, TILE_LINES, modis_tile_from_wgs84
from pyl4c.data.fixtures import EASE2_GRID_PARAMS
from pyl4c.ease2 import ease2_nested_cells
from pyl4c.epsg import EPSG
from mod17.utils import mod15a2h_qc_fail

MODIS_MOD15_DIR = '/anx_lagr4/MODIS_VIIRS/MOD15A2H/all_granules'
MODIS_MOD15_TPL = 'Y{year}/MOD15A2H.A{date}.*.hdf'
MODIS_MOD15_RX = re.compile(r'^MOD15A2H\.A(?P<date>\d{7})\.h(?P<hh>\d{2})v(?P<vv>\d{2}).*\.hdf$')
OUTPUT_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5'
DATE_START = datetime.date(2000, 1, 1)
DATE_END = datetime.date(2022, 12, 31)
N_DAYS = 8401 # Daily files from 2000-01-01 through 2022-12-31 (8401 days)
N_SUBGRID = 9 # Number of 1000-m pixels, on one side of a grid, per tower site

def main():
    'Populates the calibration driver dataset with fPAR, LAI data'
    date_list = [DATE_START]
    while date_list[-1] < DATE_END:
        date_list.append(date_list[-1] + datetime.timedelta(days = 1))
    # Get coordinate arrays for each model
    with h5py.File(OUTPUT_HDF, 'r') as hdf:
        coords = hdf['coords/lng_lat'][:].swapaxes(0, 1).tolist()
    # Get the MODIS Sinusoidal tiles that contain each site
    tile = [modis_tile_from_wgs84(pair) for pair in coords]
    tile_h, tile_v = np.array(tile).astype(np.int32).swapaxes(0,1).tolist()
    tile_h = np.array(tile_h)
    tile_v = np.array(tile_v)
    # Get the 1-km subgrid coordinates
    idx = [ # NOTE: A pixel radius of 4 yields grid side length of (2*4 + 1) = 9
        ease2_nested_cells(pair, k = 4, grid = 'M01') for pair in coords
    ]
    rows = np.array(idx)[...,0]
    cols = np.array(idx)[...,1]
    # Create output arrays
    all_fpar = np.full((N_DAYS, len(coords), N_SUBGRID**2), 255, dtype = np.uint8)
    for d, date in enumerate(tqdm(date_list)):
        file_glob = MODIS_MOD15_TPL.format(
            year = date.strftime('%Y'), date = date.strftime('%Y%j'))
        file_list = glob.glob(os.path.join(MODIS_MOD15_DIR, file_glob))
        if len(file_list) == 0:
            continue # Skip dates with no data
        # Should be approximately 290 granules for a given date
        assert len(file_list) < 300
        for filename in file_list:
            _, hh, vv = map(
                int, MODIS_MOD15_RX.match(os.path.basename(filename)).groups())
            tile_mask = np.logical_and(tile_h == hh, tile_v == vv)

            if not tile_mask.any():
                continue # Skip tiles that contain no FLUXNET sites

            ds = gdal.Open(
                f'HDF4_EOS:EOS_GRID:"{filename}":MOD_Grid_MOD15A2H:Fpar_500m')
            fpar = ds.ReadAsArray()
            qc = gdal.Open(
                f'HDF4_EOS:EOS_GRID:"{filename}":MOD_Grid_MOD15A2H:FparLai_QC')\
                    .ReadAsArray()
            # Mask out low-quality retrievals
            fail = mod15a2h_qc_fail(qc)
            fpar[fail] = 255
            wkt0 = ds.GetProjection()
            gt0 = list(ds.GetGeoTransform())
            gt = EASE2_GRID_PARAMS['M01']['geotransform']
            original = rio.open(
                'MODIS_500m', 'w+', driver = 'MEM', count = 1, dtype = np.uint8,
                width = fpar.shape[1], height = fpar.shape[0],
                crs = wkt0, transform = Affine.from_gdal(*gt0),
                nodata = ds.GetRasterBand(1).GetNoDataValue())
            original.write(fpar, 1)
            shp = EASE2_GRID_PARAMS['M01']['shape']
            result = rio.open(
                'MODIS_1km', 'w+', driver = 'MEM', count = 1, dtype = np.uint8,
                height = shp[0], width = shp[1], crs = EPSG[6933],
                transform = Affine.from_gdal(*gt),
                nodata = ds.GetRasterBand(1).GetNoDataValue())
            reproject(
                source = rio.band(original, 1),
                destination = rio.band(result, 1),
                resampling = Resampling.bilinear)
            resampled = result.read(1)
            midx = np.ravel_multi_index(
                (rows[tile_mask].ravel(), cols[tile_mask].ravel()),
                resampled.shape)
            extract = resampled.ravel()[midx]
            all_fpar[d,tile_mask] = extract\
                .reshape((extract.size // N_SUBGRID**2, N_SUBGRID**2))
    with h5py.File(OUTPUT_HDF, 'a') as hdf:
        if 'fpar' in hdf['drivers'].keys():
            del hdf['drivers/fpar0']
        dataset = hdf.create_dataset(
            'drivers/fpar0', all_fpar.shape, np.uint8, all_fpar)
        dataset.attrs['source'] = 'MODIS MOD15A2H "Fpar_500m" Collection 6.1'
        dataset.attrs['workflow'] = '20230511_0401_L4C-V7_at_Yushu_sites_MOD15A2H_extract.py'


if __name__ == '__main__':
    main()
