'''
Interpolates the 8-day MODIS MOD15A2H fPAR and LAI data so as to obtain daily
values--a forward- and backward-filling approach, no linear interpolation.
This instance calculates a climatology (and fills with climatology) *before*
interpolation.
'''

import datetime
import os
import h5py
import numpy as np
from scipy.ndimage import generic_filter
from pyl4c import suppress_warnings
from pyl4c.science import climatology365, ordinals365

OUTPUT_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5'
DATE_START = datetime.date(2000, 1, 1)
DATE_END = datetime.date(2022, 12, 31)
N_SUBGRID = 9 # Number of 1000-m pixels, on one side of a grid, per tower site

@suppress_warnings
def main():
    'Interpolates fPAR data'
    dates = [DATE_START]
    while dates[-1] < DATE_END:
        dates.append(dates[-1] + datetime.timedelta(days = 1))
    with h5py.File(OUTPUT_HDF, 'r') as hdf:
        fpar0 = hdf['drivers/fpar0'][:].astype(np.float32)
    # NOTE: valid range for both fPAR and LAI tops out at 100
    fpar_masked = np.where(
        np.logical_or(fpar0 < 0, fpar0 > 100), np.nan, fpar0)
    assert not np.apply_along_axis(
        lambda x: np.isnan(x).all(), 0, np.nanmean(fpar0, axis = -1)).any()
    # Compute fPAR and LAI climatologies
    fpar_clim = climatology365(fpar_masked, dates, ignore_leap = False)
    doy = np.array(ordinals365(dates))
    # Calculate truly missing values (as opposed to unobserved values within
    #   an 8-day interval) based on an 8-day moving window
    fpar_avg = np.nanmean(fpar_masked, axis = -1) # Average over subgrid
    filt = np.stack([ # Assumed to be the same for fPAR and LAI
        generic_filter( # size = 9 refers to our "8-day" window (needs odd num.)
            fpar_avg[:,i], lambda x: np.isnan(x).sum(), size = 9)
        for i in range(0, fpar_avg.shape[1])
    ], axis = 1)[...,np.newaxis].repeat(N_SUBGRID**2, axis = -1)
    # Fill in climatology...
    fpar = np.where(np.logical_or(filt > 8, fpar0 > 100),
        fpar_clim[doy-1], fpar0)
    # Then, replace -9999 with NaN
    fpar = np.where(fpar < 0, np.nan, fpar)
    # Forward, then backward-fill, separately for each subgrid pixel
    #   (for compatibility with non-vectorized functions)
    fpar_final = np.stack([
        fill_forward(fill_forward(fpar[...,i])[::-1])[::-1]
        for i in range(0, fpar.shape[-1])
    ], axis = 2)
    with h5py.File(OUTPUT_HDF, 'a') as hdf:
        shp = fpar_final.shape
        # Convert from fPAR units to a proportion [0,1]
        dset = hdf.create_dataset('drivers/fpar', shp, fpar_final.dtype, fpar_final / 100)
        dset.attrs['source'] = 'MODIS MOD15A2H "Fpar_500m" Collection 6.1'
        dset.attrs['description'] = 'Filled with climatology, then forward/backward interpolated (nearest neighbor) to daily time steps, then climatology calculated'
        dset.attrs['workflow'] = '20230511_0501_L4C-V7_at_Yushu_sites_MOD15A2H_interpolation.py'


def fill_forward(arr):
    'Forward filling, along axis 0, of NaNs with earlier non-NaN entires'
    mask = np.isnan(arr)
    # Get the numeric index along axis 0 where non-NaN, zero otherwise
    idx = np.where(
        ~mask, np.arange(mask.shape[0])[:,np.newaxis]\
            .repeat(arr.shape[1], axis = 1), 0)
    # Fill-forward with the last non-NaN numeric index
    np.maximum.accumulate(idx, axis = 0, out = idx)
    return arr[idx, np.arange(idx.shape[1])[None,:]]


if __name__ == '__main__':
    main()
