'''
Calculates the surface meteorology ("MET") variables required to run L4_C at
specific Western Rangelands tower sites.

    drivers/
      par
      vpd
      tsurf
      tmin
      smrz
      smsf

'''

import os
import h5py
import numpy as np
from pyl4c import suppress_warnings
from pyl4c.ease2 import ease2_from_wgs84
from pyl4c.science import vpd as vapor_pressure_deficit
from pyl4c.science import rescale_smrz

OUTPUT_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5'
LMC_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/ancillary_data/SMAP_L4_SM_lmc_00000000T000000_Vv7032_001.h5'
LMC_POROSITY_FIELD = 'Land-Model-Constants_Data/clsm_poros'

@suppress_warnings
def main():
    'Calculates the surface meteorology variables required by L4_C'
    with h5py.File(LMC_HDF, 'r') as hdf:
        porosity = hdf[LMC_POROSITY_FIELD][:]
    with h5py.File(OUTPUT_HDF, 'a') as hdf:
        coords = hdf['coords/lng_lat'][:].swapaxes(0, 1).tolist()
        idx = [ease2_from_wgs84(xy, grid = 'M09') for xy in coords]
        # Convert SW_rad from [W m-2] to [MJ m-2], then take 45%;
        #   3600 secs hr-1 times (1 MJ / 1e6 Joules) == 0.0036
        period_hrs = 1 # Radiation is measured hourly
        par = 0.45 * (0.0036 * (24 / period_hrs) * hdf['MERRA2/SWGDN'][:])
        vpd = vapor_pressure_deficit(
            hdf['MERRA2/QV2M'][:], hdf['MERRA2/PS'][:], hdf['MERRA2/T2M'][:])
        vpd = np.where(vpd < 0, 0, vpd)
        # Compute porosity to convert VWC to wetness
        midx = np.ravel_multi_index(np.array(idx).swapaxes(0, 1).tolist(), porosity.shape)
        site_porosity = porosity.ravel()[midx]
        smsf = 100 * hdf['drivers/smsf_vwc'][:] / site_porosity[np.newaxis,:]
        smrz0 = 100 * hdf['drivers/smrz_vwc'][:] / site_porosity[np.newaxis,:]
        smrz = rescale_smrz(smrz0, np.nanmin(smrz0, axis = 0))
        # Creating new datasets
        dset = hdf.create_dataset('drivers/par', par.shape, np.float32, par)
        dset.attrs['description'] = 'Photosynthetically active radiation'
        dset.attrs['units'] = 'MJ m-2 day-1'
        dset = hdf.create_dataset('drivers/vpd', vpd.shape, np.float32, vpd)
        dset.attrs['description'] = 'Vapor pressure deficit'
        dset.attrs['units'] = 'Pascals'
        dset = hdf.create_dataset(
            'drivers/tsurf', vpd.shape, np.float32, hdf['MERRA2/TS'][:])
        dset.attrs['description'] = 'Surface skin temperature'
        dset.attrs['units'] = 'K'
        dset = hdf.create_dataset(
            'drivers/tmin', vpd.shape, np.float32, hdf['MERRA2/Tmin'][:])
        dset.attrs['description'] = 'Daily minimum temperature at 2-meter height'
        dset.attrs['units'] = 'K'
        dset = hdf.create_dataset('drivers/smsf', smsf.shape, np.float32, smsf)
        dset.attrs['description'] = 'Surface soil moisture'
        dset.attrs['units'] = 'wetness (%)'
        dset = hdf.create_dataset('drivers/smrz', smrz.shape, np.float32, smrz)
        dset.attrs['description'] = 'Root-zone soil moisture'
        dset.attrs['units'] = 'wetness (%)'


if __name__ == '__main__':
    main()
