'''
Loads the state variables from the SMAP L4_C Version 7 product for specific
Western Rangelands tower sites.

    state/
      PFT
      npp_sum

'''

import os
import h5py
import numpy as np
from pyl4c import suppress_warnings
from pyl4c.spatial import as_array
from pyl4c.ease2 import ease2_from_wgs84, ease2_nested_cells

OUTPUT_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5'
L4C_PFT_MAP = '/anx_lagr3/arthur.endsley/SMAP_L4C/ancillary_data/SMAP_L4C_PFT_map_1km.tiff'
NPP_SUM_FILE = '/anx_lagr3/arthur.endsley/SMAP_L4C/calibration/v7_Y2022/delivery/tcf_OL7000_npp_sum_M01.flt32'
N_SUBGRID_RADIUS = 4 # Number of 1000-m pixels as a radius around a tower (including tower pixel)
N_SUBGRID = (N_SUBGRID_RADIUS*2 + 1)**2

@suppress_warnings
def main():
    'Extracts state information for running L4C at specific sites'
    # Load the PFT map
    pft_map, gt, wkt = as_array(L4C_PFT_MAP, band_axis = False)
    # Load the NPP map
    npp_map = np.fromfile(NPP_SUM_FILE, dtype = np.float32)\
        .reshape(pft_map.shape)
    with h5py.File(OUTPUT_HDF, 'a') as hdf:
        coords = hdf['coords/lng_lat'][:].swapaxes(0, 1).tolist()
        idx = np.array([
            ease2_nested_cells(xy, N_SUBGRID_RADIUS, grid = 'M01')
            for xy in coords
        ])
        idx = idx.reshape((idx.size // 2, 2)).swapaxes(0, 1)
        midx = np.ravel_multi_index(idx.tolist(), pft_map.shape)
        pft = pft_map.ravel()[midx].reshape((len(coords), N_SUBGRID))
        npp = npp_map.ravel()[midx].reshape((len(coords), N_SUBGRID))
        # Create new datasets
        dset = hdf.create_dataset('state/PFT', pft.shape, np.uint8, pft)
        dset.attrs['description'] = 'Plant functional type classification, based on MODIS MCD12Q1'
        dset = hdf.create_dataset('state/npp_sum', npp.shape, np.float32, npp)
        dset.attrs['description'] = 'Annual sum of net primary production (NPP)'
        dset.attrs['units'] = 'g C m-2 year-1'


if __name__ == '__main__':
    main()
