'''
Forward run of the L4C point process at Western Rangelands sites.
'''

import pickle
import datetime
import warnings
import numpy as np
import h5py
from pyl4c.science import climatology365, soc_analytical_spinup, soc_numerical_spinup, arrhenius
from pyl4c.stats import linear_constraint
from pyl4c.data.fixtures import restore_bplut
from pyl4c.apps.l4c import report
from pyl4c.apps.l4c.main import L4CForwardProcessPoint

OUTPUT_FILE_TEMPLATE = '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/20230511_L4_C_NRv10_Western_Rangelands_sites.csv'

# Update BPLUT parameters for this experiment
params_v7 = restore_bplut('/anx_lagr3/arthur.endsley/SMAP_L4C/calibration/v7_Y2022/delivery/SMAP_BPLUT_V7_rev_20220728.csv')
config = {
    'bplut': params_v7,
    'inputs_file_path': '/anx_lagr3/arthur.endsley/SMAP_L4C/applications/Western_Rangelands/L4_C_tower_site_drivers_NRv10_for_Yushu_sites.h5',
    'site_count': 64,
    'time_steps': 8401
}
dt = [
    (datetime.datetime(2000, 1, 1) + datetime.timedelta(days = d))\
        .strftime('%Y-%m-%d')
    for d in range(0, config['time_steps'])
]

# Perform soil organic carbon spin-up (if needed)
with h5py.File(config['inputs_file_path'], 'a') as hdf:
    if 'soil_organic_carbon' not in hdf['state'].keys():
        pft = hdf['state/PFT'][:]
        litterfall = hdf['state/npp_sum'][:] / 365
        dates = [
            datetime.date(y, m, d) for y, m, d in hdf['time'][:,:3].tolist()
        ]
        soil_t = hdf['drivers/tsoil'][:][...,np.newaxis]\
            .repeat(81, axis = 2)
        soil_t = climatology365(soil_t, dates)
        soil_m = hdf['drivers/smsf'][:][...,np.newaxis]\
            .repeat(81, axis = 2)
        soil_m = climatology365(soil_m, dates)
        decay_rates = params_v7['decay_rates'][:,pft]
        f_smsf = linear_constraint(
            params_v7['smsf'][0,pft], params_v7['smsf'][1,pft])(soil_m)
        f_tsoil = arrhenius(soil_t, params_v7['tsoil'][0,pft])
        fmet = params_v7['f_metabolic'][0,pft]
        fstr = params_v7['f_structural'][0,pft]
        k_mult = f_tsoil * f_smsf
        print('Performing SOC spin-up...')
        c0, c1, c2 = soc_analytical_spinup(
            litterfall, k_mult, fmet, fstr, decay_rates)
        soc_init = np.stack((c0, c1, c2))
        soc, tol = soc_numerical_spinup(
            soc_init, litterfall, k_mult, fmet, fstr, decay_rates, verbose = True)
        soc = np.array(soc)
        dset = hdf.create_dataset('state/soil_organic_carbon', soc.shape, np.float32, soc)
        dset.attrs['description'] = 'Initial soil organic carbon state, based on analytical and numerical spin-up with climatology'

with h5py.File(config['inputs_file_path'], 'r') as hdf:
    report(hdf)

with warnings.catch_warnings():
    warnings.simplefilter('ignore')
    model = L4CForwardProcessPoint(config, debug = True)
    model.run()

print('Serializing results...')
model.fluxes.serialize(OUTPUT_FILE_TEMPLATE, time_labels = dt, prec = 2)
model.state.serialize(OUTPUT_FILE_TEMPLATE, time_labels = dt)
